'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

/*************************************************  DEFAULT OPTIONS ***************************************************/

var oldLocationChangePayload = false;
var routerReducerKey = 'router';
var isReduxTravelling = false;
var showHistoryAction = false;

/*************************************************  REDUX ACTIONS *****************************************************/

var CALL_HISTORY_METHOD = '@@router/CALL_HISTORY_METHOD';
var LOCATION_CHANGE = exports.LOCATION_CHANGE = '@@router/LOCATION_CHANGE';

var locationChangeAction = function locationChangeAction(location, action) {
  return {
    type: LOCATION_CHANGE,
    payload: oldLocationChangePayload ? Object.assign({}, location, { action: action }) : { location: location, action: action }
  };
};

function updateLocation(method) {
  return function () {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return {
      type: CALL_HISTORY_METHOD,
      payload: { method: method, args: args }
    };
  };
}

var _push = updateLocation('push');
exports.push = _push;
var _replace = updateLocation('replace');
exports.replace = _replace;
var _go = updateLocation('go');
exports.go = _go;
var _goBack = updateLocation('goBack');
exports.goBack = _goBack;
var _goForward = updateLocation('goForward');

/*************************************************  REDUX REDUCER *****************************************************/

exports.goForward = _goForward;
var initialState = {
  location: null,
  action: null
};

var routerReducer = function routerReducer() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;

  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      type = _ref.type,
      payload = _ref.payload;

  if (type === LOCATION_CHANGE) {
    if (oldLocationChangePayload) {
      var _ref2 = payload || {},
          _action = _ref2.action,
          _location = _objectWithoutProperties(_ref2, ['action']);

      return Object.assign({}, state, { location: _location, action: _action });
    }

    var _ref3 = payload || {},
        location = _ref3.location,
        action = _ref3.action;

    return Object.assign({}, state, { location: location, action: action });
  }
  return state;
};

/*************************************************  REDUX MIDDLEWARE **************************************************/

// eslint-disable-next-line
exports.routerReducer = routerReducer;
var routerMiddleware = exports.routerMiddleware = function routerMiddleware(history) {
  return function () {
    return function (next) {
      return function (action) {
        if (action.type !== CALL_HISTORY_METHOD) {
          return next(action);
        }
        var _action$payload = action.payload,
            method = _action$payload.method,
            args = _action$payload.args;

        history[method].apply(history, _toConsumableArray(args));
        if (showHistoryAction) return next(action);
      };
    };
  };
};

/**********************************************  REDUX TRAVELLING  ***************************************************/

var handleReduxTravelling = function handleReduxTravelling(store, history) {
  var locationEqual = function locationEqual(loc1, loc2) {
    return loc1.pathname === loc2.pathname && loc1.search === loc2.search && loc1.hash === loc2.hash;
  };

  return store.subscribe(function () {
    var sLoc = store.getState()[routerReducerKey].location;
    var hLoc = history.location;

    if (!locationEqual(sLoc, hLoc)) {
      isReduxTravelling = true;
      history.push({ pathname: sLoc.pathname, search: sLoc.search, hash: sLoc.hash });
    }
  });
};

/******************************************  REDUX FIRST HISTORY   *****************************************************/

var createReduxHistory = exports.createReduxHistory = function createReduxHistory(store, history) {
  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

  //set options
  oldLocationChangePayload = !!options.oldLocationChangePayload;
  routerReducerKey = options.routerReducerKey || routerReducerKey;
  showHistoryAction = !!options.showHistoryAction;

  if (options.reduxTravelling) {
    handleReduxTravelling(store, history);
  }

  var registeredCallback = [];

  //init location store
  store.dispatch(locationChangeAction(history.location, history.action));

  //listen to history API
  history.listen(function (location, action) {
    if (isReduxTravelling) {
      isReduxTravelling = false;
      return;
    }
    store.dispatch(locationChangeAction(location, action));

    var state = store.getState();
    registeredCallback.forEach(function (c) {
      return c(state[routerReducerKey].location, state[routerReducerKey].action);
    });
  });

  var reduxFirstHistory = {
    createHref: history.createHref,
    push: function push() {
      return store.dispatch(_push.apply(undefined, arguments));
    },
    replace: function replace() {
      return store.dispatch(_replace.apply(undefined, arguments));
    },
    go: function go() {
      return store.dispatch(_go.apply(undefined, arguments));
    },
    goBack: function goBack() {
      return store.dispatch(_goBack.apply(undefined, arguments));
    },
    goForward: function goForward() {
      return store.dispatch(_goForward.apply(undefined, arguments));
    },

    //listen tunnel
    listen: function listen(callback) {
      if (registeredCallback.indexOf(callback) < 0) {
        registeredCallback.push(callback);
      }
      return function () {
        registeredCallback = registeredCallback.filter(function (c) {
          return c !== callback;
        });
      };
    }
  };

  //location tunnel
  Object.defineProperty(reduxFirstHistory, 'location', {
    get: function get() {
      return store.getState()[routerReducerKey].location;
    }
  });

  //action tunnel
  Object.defineProperty(reduxFirstHistory, 'action', {
    get: function get() {
      return store.getState()[routerReducerKey].action;
    }
  });

  return reduxFirstHistory;
};